import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Write a description of class RandomMaze here.
 * 
 * @author (your name) 
 * @version (a version number or a date)
 */
public class RandomMaze extends World
{
    final public static int
      ADJOIN_N=1, ADJOIN_E=2, ADJOIN_S=4, ADJOIN_W=8;

    final public static int
      WIDTH=15, HEIGHT=15;

    final public static int
      FLOOR = 0,
      WALL  = 1;

    static public RandomMaze instance;

    public GreenfootImage[] robot_images;
    public GreenfootImage[] wall_images;
    public int[][] map;

    /**
     * Constructor for objects of class RandomMaze.
     * 
     */
    public RandomMaze()
    {
        // Create a new world with 15x15 cells with a cell size of 32x32 pixels.
        super(15, 15, 32);

        instance = this;

        setBackground("floor.png"); 
        
        // load the wall graphics
        wall_images = new GreenfootImage[16];
        load_wall( 0, "wall.png" );
        load_wall( ADJOIN_E, "wall-e.png" );
        load_wall( ADJOIN_E|ADJOIN_W, "wall-ew.png" );
        load_wall( ADJOIN_W, "wall-w.png" );

        load_wall( ADJOIN_S, "wall-s.png" );
        load_wall( ADJOIN_S|ADJOIN_E, "wall-es.png" );
        load_wall( ADJOIN_W|ADJOIN_S|ADJOIN_E, "wall-esw.png" );
        load_wall( ADJOIN_W|ADJOIN_S, "wall-sw.png" );

        load_wall( ADJOIN_N|ADJOIN_S, "wall-ns.png" );
        load_wall( ADJOIN_N|ADJOIN_S|ADJOIN_E, "wall-nes.png" );
        load_wall( ADJOIN_N|ADJOIN_S|ADJOIN_E|ADJOIN_W, "wall-nesw.png" );
        load_wall( ADJOIN_N|ADJOIN_S|ADJOIN_W, "wall-nsw.png" );

        load_wall( ADJOIN_N, "wall-n.png" );
        load_wall( ADJOIN_N|ADJOIN_E, "wall-ne.png" );
        load_wall( ADJOIN_N|ADJOIN_E|ADJOIN_W, "wall-new.png" );
        load_wall( ADJOIN_N|ADJOIN_W, "wall-nw.png" );

        robot_images = new GreenfootImage[4];
        robot_images[0] = new GreenfootImage( "robot-n.png" );
        robot_images[1] = new GreenfootImage( "robot-e.png" );
        robot_images[2] = new GreenfootImage( "robot-s.png" );
        robot_images[3] = new GreenfootImage( "robot-w.png" );

        map = new int[WIDTH][HEIGHT];
        createMaze();

    }

    protected void load_wall( int adjoining, String filename )
    {
      wall_images[adjoining] = new GreenfootImage(filename);
    }

    public int get( int i, int j )
    {
      if ( !isInBounds(i,j) ) return FLOOR;
      return map[i][j];
    }

    public boolean isInBounds( int i, int j )
    {
      return (i >= 0 && j >= 0 && i < WIDTH && j < HEIGHT);
    }


    public void createMaze()
    {
      // fill the maze with solid wall
      for (int j=0; j<HEIGHT; j++)
      {
        for (int i=0; i<WIDTH; i++)
        {
          map[i][j] = WALL;
        }
      }

      // burrow through the walls, taking random turns,
      // always stopping just shy of breaking through to
      // somewhere we've already been and instead retracing
      // our steps to try going a different direction
      visit( 1, 1 );

      // set the graphic of each wall based on the walls
      // around it
      for (int j=0; j<HEIGHT; j++)
      {
        for (int i=0; i<WIDTH; i++)
        {
          if (get(i,j) == WALL)
          {
            Wall wall = new Wall();
            int adjoining = 0;
            if (get(i,j-1) == WALL) adjoining |= ADJOIN_N;
            if (get(i,j+1) == WALL) adjoining |= ADJOIN_S;
            if (get(i+1,j) == WALL) adjoining |= ADJOIN_E;
            if (get(i-1,j) == WALL) adjoining |= ADJOIN_W;
            wall.setImage( wall_images[adjoining] );
            addObject( wall, i, j );
          }
        }
      }
    }

    protected void visit( int i, int j )
    {
      // set this location to be a floor tile
      map[i][j] = FLOOR;

      // randomize a list of 4 directions
      int[][] dirs = { {1,0}, {0,1}, {-1,0}, {0,-1} };
      for (int index=0; index<4; index++)
      {
        int r = (int)(Math.random() * 4);
        int[] temp = dirs[index];
        dirs[index] = dirs[r];
        dirs[r] = temp;
      }

      // consider going in each direction
      for (int[] offset : dirs)
      {
        int dx = offset[0];
        int dy = offset[1];
        if (get(i+dx*2,j+dy*2) == WALL)
        {
          // if we haven't been there, break through to that
          // spot
          map[i+dx][j+dy] = FLOOR;
          visit(i+dx*2,j+dy*2);
        }
      }
    }

}

